	**************************************************************
	*** MASTER SCRIPT * Donati, Rao, Orozco, Munoz-Boudet 2024 ***
	**************************************************************

	* This master script imports Input files (raw data), processes, cleans and combines them into 
	* Intermediate files (processed data), and eventually produces the output tables and figures used in the paper

	* Stata version 15 MP for Macintosh system

	* Set mat size
	set matsize 10000
	set varabbrev on

	* Set graph scheme
	graph set window fontface "Times New Roman"
	set scheme s1mono

	/* PERSONALIZE DIRECTORY PATH (TO BE CHANGED BY REVIEWER SO THAT IT MATCHES THE DIRECTORY OF THE REPRODUCIBILITY PACKAGE) */
	global directory "C:/Users/wb558768/Downloads/RR_IND_2024_180"
	
	/* SET DIRECTORY (NOT TO BE CHANGED) */
	cd "$directory/ReproducibilityPackage"

	/* FOLDER CONTAINING THE OUTPUT FILES (NOT TO BE CHANGED)*/
	global output "03_Output"
	
	sysdir set PLUS "00_Scripts/ado"

	/* SCRIPTS FOR RAW DATA PROCESSING and CLEANING: */
	* Each of the following scripts takes the raw panel, cross-sectional and individual-study survey data, and the subdistrict-level malaria data,
	* construct the variables to be used, and export the processed datasets
	clear all

	do 00_Scripts/01_Clean_panel.do
	do 00_Scripts/02_Clean_panel_over_time.do
	do 00_Scripts/03_Clean_xsection.do
	do 00_Scripts/04_Clean_endline.do
	do 00_Scripts/05_Clean_individual.do
	do 00_Scripts/06_Clean_subdistricts.do


	/* SCRIPTS TO REPRODUCE TABLES (as numbered in the paper): */
	* Each of the following scripts produces the respective table as numbered in the paper

	* Table 1
	do 00_Scripts/Table1.do

	* Table 2
	do 00_Scripts/Table2.do

	* Table 3
	do 00_Scripts/Table3.do

	* Table 4
	do 00_Scripts/Table4.do

	* Table 5
	do 00_Scripts/Table5.do

	* Table 6
	do 00_Scripts/Table6.do

	* Table 7
	do 00_Scripts/Table7.do

	* Table 8
	do 00_Scripts/Table8.do


	/* SCRIPTS TO REPRODUCE FIGURES (as numbered in the paper): */
	* Each of the following scripts produces the respective figure as numbered in the paper

	* Figure 3a
	do 00_Scripts/Figure3a.do

	* Figure 3b
	do 00_Scripts/Figure3b.do

	* Figure 4a
	do 00_Scripts/Figure4a.do

	* Figure 4b
	do 00_Scripts/Figure4b.do

	* Figure 5
	do 00_Scripts/Figure5.do


	* ANNEX TABLES *
	
	* Table D1
	do 00_Scripts/TableD1.do

	* Table D2
	do 00_Scripts/TableD2.do

	* Table D3
	do 00_Scripts/TableD3.do

	* Table D4
	do 00_Scripts/TableD4.do

	* Table D5
	do 00_Scripts/TableD5.do

	* Table D6
	do 00_Scripts/TableD6.do

	* Table D7
	do 00_Scripts/TableD7.do

	* Table D8
	do 00_Scripts/TableD8.do

	* Table D9
	do 00_Scripts/TableD9.do

	* Table D10
	do 00_Scripts/TableD10.do

	* Table D11
	do 00_Scripts/TableD11.do

	* Table D12
	do 00_Scripts/TableD12.do

	* Table D13
	do 00_Scripts/TableD13.do

	* Table D14
	do 00_Scripts/TableD14.do

	* Table D15
	do 00_Scripts/TableD15.do

	* Table D16
	do 00_Scripts/TableD16.do
	
	* Table D17
	do 00_Scripts/TableD17.do
	
	* Table D18
	do 00_Scripts/TableD18.do

	* Table D19
	do 00_Scripts/TableD19.do

	* Table D20
	do 00_Scripts/TableD20.do

	* Table D21
	do 00_Scripts/TableD21.do

	* Table D22
	do 00_Scripts/TableD22.do

	* Table D23
	do 00_Scripts/TableD23.do

	* Table D24
	do 00_Scripts/TableD24.do

	* Table D25
	do 00_Scripts/TableD25.do


	* ANNEX FIGURE *
	
	* Figure C3
	do 00_Scripts/FigureC3.do


	

