* Clean raw panel data
import delim using "01_RawData/full_panel.csv", clear stringc(10)
sort userid wave

*keep first response from each individual
bys userid wave (starttime): keep if _n==1

* get state name from auxiliary geography dataset
preserve
import delim using "01_RawData/base-cities.csv", clear
keep disthash state 
rename disthash stratumid
bys state stratumid: keep if _n==1

* drop district with duplicated name: Balrampur
drop if stratumid == "ff099f99"
tempfile temp
save `temp'
restore
merge m:1 stratumid using `temp', keep(master matched) nogen


* fix population that is missing in two districts (from census)
destring cluster_population, i("NA") replace
replace cluster_population = 2192933 if stratumid=="e5b6ea7f" //  district == "Sambhal"
replace cluster_population = 1274815 if stratumid=="9a766899" // district == "Samli"
gen cluster_lpopulation=log(cluster_population)


*export cluster information for other datasets
preserve
bys stratumid: keep if _n==1
keep stratumid cluster_kutcha cluster_pucca cluster_university cluster_unemployed cluster_malaria5year cluster_malaria2weeks cluster_sleepundernet cluster_population cluster_lpopulation cluster_current_total cluster_cost_per_completion cluster_cost_per_message cluster_ctr cluster_cpm state
save "01_RawData/cluster_info.dta", replace
restore

* gender
gen female2 = (gender=="Female")


* Clean time variant vars
encode worried_covid, gen(worried_covid_code)
drop worried_covid
gen not_worried_covid = (worried_covid_code==3)
gen some_worried_covid = (worried_covid_code==4)
gen worried_covid = (worried_covid_code==5)
gen ext_worried_covid = (worried_covid_code==1)
recode not_worried_covid  some_worried_covid  worried_covid  ext_worried_covid (*=.) if worried_covid_code== 2
drop worried_covid_code

encode worried_malaria, gen(worried_malaria_code)
drop worried_malaria
gen not_worried_malaria = (worried_malaria_code==3)
gen some_worried_malaria = (worried_malaria_code==4)
gen worried_malaria = (worried_malaria_code==5)
gen ext_worried_malaria = (worried_malaria_code==1)
recode not_worried_malaria  some_worried_malaria  worried_malaria  ext_worried_malaria (*=.) if worried_malaria_code== 2
drop worried_malaria_code

* use of spray
gen used_spray_yesno = (used_spray=="Yes")
replace used_spray_yesno = . if used_spray=="NA"
drop used_spray
rename used_spray_yesno used_spray


foreach x of varlist admalaria sleepundernet perc_membersbednet membersbednet longsleeves malaria2weeks fever2weeks malaria5year not_worried_covid some_worried_covid worried_covid ext_worried_covid not_worried_malaria some_worried_malaria worried_malaria ext_worried_malaria   {
     destring `x', replace i("NA")
}

recode admalaria (*=.) if wave!= 9

gen knowmnm_yesno = (knowmnm=="Yes")
replace knowmnm_yesno = . if knowmnm=="NA"
drop knowmnm
rename knowmnm_yesno knowmnm


gen admnm_yesno = (admnm=="Yes")
replace admnm_yesno = . if admnm=="NA"
drop admnm
rename admnm_yesno admnm



* Clean time invariant vars
encode distancemedicalcenter, gen(distance)
gen distance_15 = (distance==3)
gen distance_15_30 = (distance==1)
gen distance_30_60 = (distance==2)
recode distance_15 distance_15_30 distance_30_60 (*=.) if distancemedicalcenter == "NA"
drop distancemedicalcenter distance

encode religion, gen(religion_code)
gen hindu = (religion_code==3)
gen muslim = (religion_code==6)
recode hindu muslim (*=.) if religion=="NA"
drop religion*

*encode caste, gen(caste_code)

gen caste_general = (caste=="General")
gen caste_obc = (caste=="OBC")
gen caste_other = (caste=="Other")
gen caste_scdalit = (caste=="SC/Dalit")
gen caste_st = (caste=="ST")
recode caste_general caste_obc caste_other caste_scdalit caste_st (*=.) if caste == "NA"

gen secondary = (education =="Secondary")
gen primary = (education =="Primary")
recode secondary primary (*=.) if education == "NA"

foreach x of varlist age familymembers pucca kutcha semipucca university secondary primary unemployed student female generalcaste forwardcaste backwardcaste hasmosquitonet  hasairconditioning  pregnantwoman  {
     destring `x', replace i("NA")
}


gen test_malaria = (testmalaria=="Yes")
recode test_malaria (*=.) if testmalaria=="" | testmalaria=="NA"

gen help24_malaria = (timeseekhelpmalaria=="Less than 24 hours")
recode help24_malaria (*=.) if timeseekhelpmalaria =="" | timeseekhelpmalaria =="NA"

gen help_fever = (seekhelpfever=="1")
recode help_fever (*=.) if seekhelpfever=="" | seekhelpfever=="NA"

gen help24_fever = (timeseekhelpfever=="Less than 24 hours")
recode help24_fever (*=.) if timeseekhelpfever =="" | timeseekhelpfever =="NA"



* Drop respondents who rushed through the survey 
destring answer_time_90 answer_time_75 answer_time_min, replace i("NA")
keep if answer_time_90/answer_time_min >= 3 & answer_time_90/answer_time_min!=.


foreach x of varlist some_worried_malaria not_worried_malaria worried_malaria ext_worried_malaria not_worried_covid some_worried_covid worried_covid ext_worried_covid malaria2weeks malaria5year sleepundernet perc_membersbednet longsleeves used_spray fever2weeks test_malaria help_fever {
     gen base_`x' = `x' if wave == 0
}
recode base_sleepundernet base_perc_membersbednet (*=.) if hasmosquitonet==0

recode caste_general caste_obc caste_other caste_scdalit caste_st distance_15 distance_15_30  distance_30_60 hindu muslim age familymembers pucca kutcha semipucca university unemployed student female generalcaste forwardcaste backwardcaste hasmosquitonet  hasairconditioning  pregnantwoman  (*=.) if wave>0

bysort userid (wave) : carryforward base_* caste_general caste_obc caste_other caste_scdalit caste_st distance_15 distance_15_30  distance_30_60 hindu muslim age familymembers pucca kutcha semipucca university unemployed student female generalcaste forwardcaste backwardcaste hasmosquitonet  hasairconditioning  pregnantwoman, replace



* Clean data and define baseline vs followup
* drop if age is outside the 18-90 range
* drop if family members is above 30
* drop if dwelling type is unkown 

keep if age < 90 & age > 17 
keep if familymembers < 30
keep if pucca!=.

gen start = date(survey_start_time, "YMDhms")
format start %td
gen end = date(survey_end_time, "YMDhms")
format end %td

* drop if they completed the panel after Jan 31 2021
keep if end <= 22311 //2021-01-31

* post dummy
gen post = (start > 22158) //2020-08-31

recode post (0=.) if end >= 22146 //2020-08-19

drop if post == .
drop if wave == 1 & post == 0
drop waves_answered

* generate dummy which idetnifies naive vs optimized recruitment
gen tmp = (start<22121) //2020-07-25
bys userid: egen naive_recruitment= max(tmp)
recode naive_recruitment (*=.) if post == 1
drop tmp 

gen tmp = 1
by userid: egen tot_waves_answered = total(tmp)

gen waves_answered_all = 1
gen waves_answered_net = 1 if sleepundernet!=.  & perc_membersbednet!=.
gen waves_answered_spray = 1 if used_spray!=.
gen waves_answered_sleeves = 1 if longsleeves!=.
gen waves_answered_malaria2weeks = 1 if malaria2weeks!=.
gen waves_answered_test_malaria = 1 if test_malaria!=.
gen waves_answered_fever2weeks = 1 if fever2weeks!=.
gen waves_answered_help24_malaria = 1 if help24_malaria !=.
gen waves_answered_help24_fever = 1 if help24_fever!=.

sort userid wave
collapse (sum) waves_answered_* (mean) avg_perc_membersbednet=perc_membersbednet avg_sleepundernet=sleepundernet avg_longsleeves=longsleeves avg_usedspray=used_spray (max) end malaria2weeks test_malaria help24_malaria fever2weeks help_fever help24_fever  (first) tot_waves_answered shortcode state treatment stratumid age familymembers pucca kutcha semipucca university secondary primary unemployed student female caste_* forwardcaste backwardcaste hasmosquitonet distance_15 distance_15_30 distance_30_60 hasairconditioning  pregnantwoman hindu muslim base_* cluster_* (firstnm) naive_recruitment admnm knowmnm admalaria, by(userid post)

encode state, gen(state_id)

egen semipucca_kutcha = rowmax(kutcha semipucca)

egen base_malaria_riskperception = rowmax(base_worried_malaria base_ext_worried_malaria)
egen base_covid_riskperception = rowmax(base_worried_covid base_ext_worried_covid)

gen treatment_pucca = treatment*pucca 

* label variables
la var treatment "T"
la var pucca "Solid house"
la var treatment_pucca "T $\times$ Solid house"
 
la var waves_answered_all "N. of survey waves per respondent"
la var tot_waves_answered "N. of total survey waves per respondent"
la var age "Age"
la var semipucca "Semi-solid house"
la var semipucca_kutcha "Non/Semi-solid house"
la var familymembers "N. of HH members"
la var university "Has completed university"
la var secondary "Has completed secondary school"
la var primary "Has completed primary school"
la var unemployed "Unemployed"
la var student "Student"
la var female "Female"
la var caste_general "General caste"
la var caste_obc "OBC caste"
la var caste_scdalit "SC/Dalit caste"
la var caste_st "ST caste"
la var distance_15 "Medical center distance < 15min"
la var distance_15_30 "Medical center distance 15-30min"
la var distance_30_60 "Medical center distance 30-60min"
la var hasairconditioning "Has air conditioning"
la var hasmosquitonet "Has mosquito net"
la var pregnantwoman "Pregnant woman in house"
la var hindu "Hindu"
la var muslim "Muslim"
la var base_malaria5year "Any member had malaria in last 5 years" 
la var base_not_worried_malaria "Not worried about malaria"
la var base_some_worried_malaria "Somewhat worried about malaria"
la var base_worried_malaria "Worried about malaria"
la var base_ext_worried_malaria "Extremely worried about malaria"
la var base_not_worried_covid "Not worried about COVID-19"
la var base_some_worried_covid "Somewhat worried about COVID-19"
la var base_worried_covid "Worried about COVID-19"
la var base_ext_worried_covid "Extremely worried about COVID-19"
la var base_malaria_riskperception "Worried/Very worried about malaria"
la var base_covid_riskperception "Worried/Very worried about COVID-19"

la var base_sleepundernet "Used bednet the previous night"
la var base_perc_membersbednet "Share of HH members who used bednet the previous night"
la var base_longsleeves "Used long-sleeve clothing the previous day"
la var base_used_spray "Used body/wall sprays the previous day"
la var base_malaria2weeks "Any member had malaria in last 2 weeks"
la var base_fever2weeks "Any member had a fever in last 2 weeks"
la var base_test_malaria "Any member tested for malaria"
la var base_help_fever "Any member sought medical help if fever"
la var naive_recruitment "Recruited during naive advertising"
replace cluster_population = cluster_population/1000000

* convert costs from INR to USD
gen cluster_cost_per_completion_usd = cluster_cost_per_completion*0.012
gen cluster_cost_per_message_usd = cluster_cost_per_message*0.012
gen cluster_cpm_usd = cluster_cpm*0.012

la var cluster_population "Population (Million)"
la var cluster_pucca "Share living in solid houses"
la var cluster_sleepundernet "Share sleeping under net"
la var cluster_university "Share with university degree"
la var cluster_unemployed "Share unemployed"
la var cluster_malaria2weeks "Share with malaria in last 2 weeks"
la var cluster_malaria5year "Share with malaria in last 5 years"
la var cluster_cost_per_completion_usd "Average ad cost per survey completed (USD)"   
la var cluster_cost_per_message_usd "Average ad cost per survey started (USD)"   
la var cluster_ctr "Average click-through rate (\%)"
la var cluster_cpm_usd "Average cost per mille (USD)"

drop if missing(university) 

save "02_ProcessedData/full_panel_processed.dta", replace



