* Clean raw cross sectional data
import delim using "01_RawData/xsection.csv", clear stringc(11)
sort userid wave


* import cluster-level controls
drop cluster_*
drop if stratumid==""
merge m:1 stratumid using "01_RawData/cluster_info.dta", keep(master matched) nogen

* generate variables
gen female2 = (gender=="Female")

gen self_employed = (occupation=="Self-employed")
recode self_employed (*=.) if occupation=="NA"

* Outcomes
encode worriedcovid, gen(worried_covid_code)
drop worriedcovid
gen not_worried_covid = (worried_covid_code==3)
gen some_worried_covid = (worried_covid_code==4)
gen worried_covid = (worried_covid_code==5)
gen ext_worried_covid = (worried_covid_code==1)
recode not_worried_covid  some_worried_covid  worried_covid  ext_worried_covid (*=.) if worried_covid_code== 2
drop worried_covid_code

encode worriedmalaria, gen(worried_malaria_code)
drop worriedmalaria
gen not_worried_malaria = (worried_malaria_code==3)
gen some_worried_malaria = (worried_malaria_code==4)
gen worried_malaria = (worried_malaria_code==5)
gen ext_worried_malaria = (worried_malaria_code==1)
recode not_worried_malaria  some_worried_malaria  worried_malaria  ext_worried_malaria (*=.) if worried_malaria_code== 2
drop worried_malaria_code


gen test_malaria = (testmalaria=="Yes")
recode test_malaria (*=.) if testmalaria=="" | testmalaria=="NA"

gen help24_malaria = (timeseekhelpmalaria=="Less than 24 hours")
recode help24_malaria (*=.) if timeseekhelpmalaria =="" | timeseekhelpmalaria =="NA"


gen help_fever = (seekhelpfever=="1")
recode help_fever (*=.) if seekhelpfever=="" | seekhelpfever=="NA"

gen help24_fever = (timeseekhelpfever=="Less than 24 hours")
recode help24_fever (*=.) if timeseekhelpfever =="" | timeseekhelpfever =="NA"



gen would_buy_net = (buymosquitonet=="Yes")
recode would_buy_net (*=.) if buymosquitonet =="NA" | buymosquitonet=="Don't know"

gen know_buy_net = (knowbuymosquitonet=="Yes")
recode know_buy_net (*=.) if knowbuymosquitonet =="NA"

foreach x of varlist malaria4months fever4months numbermalaria4month  hasmosquitonet  {
     destring `x', replace i("NA")
}


destring numbermosquitonet, replace i("NA" ",") force


gen pay_for_net = regexs(2) if regexm(paymosquitonet, "^([^0-9]*)([0-9]+)([^0-9]*)$")
destring pay_for_net, replace
sum pay_for_net,d
replace pay_for_net = . if pay_for_net>r(p99)


gen medical_center_dh = (typemedicalcenter=="District Hospital")
gen medical_center_chc = (typemedicalcenter=="Community Health Center")
gen medical_center_phc = (typemedicalcenter=="Primary Health Center")
gen medical_center_sc = (typemedicalcenter=="Sub Center")
recode medical_center_dh medical_center_chc medical_center_phc medical_center_sc (*=.) if typemedicalcenter=="NA"

gen treat_net = (treatmosquitonet=="Yes")
recode treat_net (*=.) if treatmosquitonet =="NA" | treatmosquitonet =="Donât know" | treatmosquitonet=="DonÃ¢ÂÂt know"


* Time invariant vars
encode distancemedicalcenter, gen(distance)
gen distance_15 = (distance==3)
gen distance_15_30 = (distance==1)
gen distance_30_60 = (distance==2)
recode distance_15 distance_15_30 distance_30_60 (*=.) if distancemedicalcenter=="NA"
drop distancemedicalcenter distance

encode religion, gen(religion_code)
gen hindu = (religion_code==3)
gen muslim = (religion_code==5)
recode hindu muslim (*=.) if religion=="NA"
drop religion*

encode caste, gen(caste_code)

gen caste_general = (caste=="General")
gen caste_obc = (caste=="OBC")
gen caste_other = (caste=="Other")
gen caste_scdalit = (caste=="SC/Dalit")
gen caste_st = (caste=="ST")
recode caste_general caste_obc caste_other caste_scdalit caste_st (*=.) if caste == "NA"

gen secondary = (education =="Secondary")
gen primary = (education =="Primary")
recode secondary primary (*=.) if education == "NA"

foreach x of varlist age familymembers pucca kutcha semipucca university secondary primary unemployed student female generalcaste forwardcaste backwardcaste hasmosquitonet  hasairconditioning  pregnantwoman  {
     destring `x', replace i("NA")
}



destring answer_time_median answer_time_min, replace i("NA")
keep if answer_time_90/answer_time_min >= 3 & answer_time_90/answer_time_min!=.

* Clean data and define baseline vs followup
* drop if age is outside the 18-90 range
* drop if family members is above 30
* drop if dwelling type is unkown 
keep if age < 90 & age > 17 
keep if familymembers < 30
keep if pucca!=.


gen start = date(survey_start_time, "YMDhms")
format start %td
gen end = date(survey_end_time, "YMDhms")
format end %td

gen duration_min = survey_duration/60
gen duration_h = duration_min/60 
gen duration_days = duration_h/24

keep if end <= 22370 //2021-03-31


encode state, gen(state_id)

gen treatment_pucca = treatment*pucca 

recode numbermosquitonet (.=0) if hasmosquitonet==0

gen inv_cpm = 1/cluster_cpm
gen treatment_intensity = treatment*inv_cpm

la var treatment "T"
la var pucca "Solid house"
la var treatment_pucca "T $\times$ Solid house"
 
gen lpay_for_net = log(pay_for_net+1)

recode numbermalaria4months (.=0) if malaria4months==0
gen lnumbermalaria4months = log(numbermalaria4months+1)

gen share_malaria = numbermalaria4months/familymembers


* Trim number of mosquito nets to remove outliers
sum numbermosquitonet, d
recode numbermosquitonet (*=.) if numbermosquitonet>r(p99)
recode share_malaria malaria4months numbermalaria4months lnumbermalaria4months test_malaria  (*=.) if share_malaria>1


gen net_percapita = numbermosquitonet/familymembers
gen lnumbermosquitonet = log(numbermosquitonet+1)
egen semipucca_kutcha = rowmax(kutcha semipucca)


* Label variables

la var pucca "Solid house"

la var age "Age"
la var semipucca "Semi-solid house"
la var semipucca_kutcha "Non/Semi-solid house"
la var familymembers "N. of HH members"
la var university "Has completed university"
la var secondary "Has completed secondary school"
la var primary "Has completed primary school"
la var unemployed "Unemployed"
la var student "Student"
la var female "Female"
la var caste_general "General caste"
la var caste_obc "OBC caste"
la var caste_scdalit "SC/Dalit caste"
la var caste_st "ST caste"
la var distance_15 "Medical center distance < 15min"
la var distance_15_30 "Medical center distance 15-30min"
la var distance_30_60 "Medical center distance 30-60min"
la var hasairconditioning "Has air conditioning"
la var hasmosquitonet "Has mosquito net"
la var pregnantwoman "Pregnant woman in house"
la var hindu "Hindu"
la var muslim "Muslim"
la var net_percapita "Mosquito nets per HH member"

replace cluster_population = cluster_population/1000000
gen cluster_cost_per_completion_usd = cluster_cost_per_completion*0.012
gen cluster_cost_per_message_usd = cluster_cost_per_message*0.012
gen cluster_cpm_usd = cluster_cpm*0.012


la var cluster_population "Population (Million)"
la var cluster_pucca "Share living in solid houses"
la var cluster_sleepundernet "Share sleeping under net"
la var cluster_university "Share with university degree"
la var cluster_unemployed "Share unemployed"
la var cluster_malaria2weeks "Share with malaria in last 2 weeks"
la var cluster_malaria5year "Share with malaria in last 5 years"
la var cluster_cost_per_completion_usd "Average ad cost per survey completed (USD)"   
la var cluster_cost_per_message_usd "Average ad cost per survey started (USD)"   
la var cluster_ctr "Average click-through rate (\%)"
la var cluster_cpm_usd "Average cost per mille (USD)"

drop if missing(university, share_malaria, hasmosquitonet, hasairconditioning) 

save "02_ProcessedData/xsection_processed.dta", replace



