* Extract endline from panel data and clean it
import delim using "01_RawData/full_panel.csv", clear stringc(10)
sort userid wave

*keep first response from each individual
bys userid wave (starttime): keep if _n==1

preserve
import delim using "01_RawData/base-cities.csv", clear
keep disthash state 
rename disthash stratumid
bys state stratumid: keep if _n==1

* drop district with duplicated name: Balrampur
drop if stratumid == "ff099f99"
tempfile temp
save `temp'
restore
merge m:1 stratumid using `temp', keep(master matched) nogen

destring cluster_population, i("NA") replace
replace cluster_population = 2192933 if stratumid=="e5b6ea7f" //  district == "Sambhal"
replace cluster_population = 1274815 if stratumid=="9a766899" // district == "Samli"
gen cluster_lpopulation=log(cluster_population)

gen female2 = (gender=="Female")


* Time variant vars
encode worried_covid, gen(worried_covid_code)
drop worried_covid
gen not_worried_covid = (worried_covid_code==3)
gen some_worried_covid = (worried_covid_code==4)
gen worried_covid = (worried_covid_code==5)
gen ext_worried_covid = (worried_covid_code==1)
recode not_worried_covid  some_worried_covid  worried_covid  ext_worried_covid (*=.) if worried_covid_code== 2
drop worried_covid_code

encode worried_malaria, gen(worried_malaria_code)
drop worried_malaria
gen not_worried_malaria = (worried_malaria_code==3)
gen some_worried_malaria = (worried_malaria_code==4)
gen worried_malaria = (worried_malaria_code==5)
gen ext_worried_malaria = (worried_malaria_code==1)
recode not_worried_malaria  some_worried_malaria  worried_malaria  ext_worried_malaria (*=.) if worried_malaria_code== 2
drop worried_malaria_code

* Error in the variable: need to export/import again
gen used_spray_yesno = (used_spray=="Yes")
replace used_spray_yesno = . if used_spray=="NA"
drop used_spray
rename used_spray_yesno used_spray



foreach x of varlist admalaria sleepundernet perc_membersbednet membersbednet longsleeves malaria2weeks fever2weeks malaria5year not_worried_covid some_worried_covid worried_covid ext_worried_covid not_worried_malaria some_worried_malaria worried_malaria ext_worried_malaria   {
     destring `x', replace i("NA")
}

recode admalaria (*=.) if wave!= 9

gen knowmnm_yesno = (knowmnm=="Yes")
replace knowmnm_yesno = . if knowmnm=="NA"
drop knowmnm
rename knowmnm_yesno knowmnm


gen admnm_yesno = (admnm=="Yes")
replace admnm_yesno = . if admnm=="NA"
drop admnm
rename admnm_yesno admnm



* Time invariant vars
encode distancemedicalcenter, gen(distance)
gen distance_15 = (distance==3)
gen distance_15_30 = (distance==1)
gen distance_30_60 = (distance==2)
recode distance_15 distance_15_30 distance_30_60 (*=.) if distancemedicalcenter == "NA"
drop distancemedicalcenter distance

encode religion, gen(religion_code)
gen hindu = (religion_code==3)
gen muslim = (religion_code==6)
recode hindu muslim (*=.) if religion=="NA"
drop religion*

encode caste, gen(caste_code)

gen caste_general = (caste=="General")
gen caste_obc = (caste=="OBC")
gen caste_other = (caste=="Other")
gen caste_scdalit = (caste=="SC/Dalit")
gen caste_st = (caste=="ST")
recode caste_general caste_obc caste_other caste_scdalit caste_st (*=.) if caste == "NA"

gen secondary = (education =="Secondary")
gen primary = (education =="Primary")
recode secondary primary (*=.) if education == "NA"

foreach x of varlist age familymembers pucca kutcha semipucca university secondary primary unemployed student female generalcaste forwardcaste backwardcaste hasmosquitonet  hasairconditioning  pregnantwoman  {
     destring `x', replace i("NA")
}


* Drop respondents who rushed through the survey 
destring answer_time_90 answer_time_75 answer_time_min, replace i("NA")
keep if answer_time_90/answer_time_min >= 3 & answer_time_90/answer_time_min!=.


foreach x of varlist some_worried_malaria not_worried_malaria worried_malaria ext_worried_malaria not_worried_covid some_worried_covid worried_covid ext_worried_covid malaria2weeks malaria5year {
     gen base_`x' = `x' if wave == 0
}


recode caste_general caste_obc caste_other caste_scdalit caste_st distance_15 distance_15_30  distance_30_60 hindu muslim age familymembers pucca kutcha semipucca university unemployed student female generalcaste forwardcaste backwardcaste hasmosquitonet  hasairconditioning  pregnantwoman  (*=.) if wave>0

bysort userid (wave) : carryforward base_* caste_general caste_obc caste_other caste_scdalit caste_st distance_15 distance_15_30  distance_30_60 hindu muslim age familymembers pucca kutcha semipucca university unemployed student female generalcaste forwardcaste backwardcaste hasmosquitonet  hasairconditioning  pregnantwoman, replace



* Clean data and define baseline vs followup
keep if age < 90 & age > 17 
keep if familymembers < 30
keep if pucca!=.

gen start = date(survey_start_time, "YMDhms")
format start %td
gen end = date(survey_end_time, "YMDhms")
format end %td

* Keep endline
keep if wave == 9 & start <= 22311 //2021-01-31


sort userid wave
keep shortcode state treatment stratumid age used_spray familymembers pucca kutcha semipucca university secondary primary unemployed student female caste_* forwardcaste backwardcaste hasmosquitonet distance_15 distance_15_30 distance_30_60 hasairconditioning  pregnantwoman hindu muslim base_* cluster_* admnm knowmnm admalaria

encode state, gen(state_id)

egen base_malaria_riskperception = rowmax(base_worried_malaria base_ext_worried_malaria)
egen base_covid_riskperception = rowmax(base_worried_covid base_ext_worried_covid)

gen treatment_pucca = treatment*pucca 
la var treatment "T"
la var pucca "Solid house"
la var treatment_pucca "T $\times$ Solid house"
 
la var age "Age"
la var semipucca "Semi-solid house"
la var familymembers "N. of HH members"
la var university "Has completed university"
la var secondary "Has completed secondary school"
la var primary "Has completed primary school"
la var unemployed "Unemployed"
la var student "Student"
la var female "Female"
la var caste_general "General caste"
la var caste_obc "OBC caste"
la var caste_scdalit "SC/Dalit caste"
la var caste_st "ST caste"
la var distance_15 "HMedical center distance < 15min"
la var distance_15_30 "Medical center distance 15-30min"
la var distance_30_60 "Medical center distance 30-60min"
la var hasairconditioning "Has air conditioning"
la var hasmosquitonet "Has mosquito net"
la var pregnantwoman "Pregnant woman in house"
la var hindu "Hindu"
la var muslim "Muslim"
la var base_malaria2weeks "Had malaria in last 2 weeks"
la var base_malaria5year "Had malaria in last 5 years" 
la var base_not_worried_malaria "Not worried about malaria"
la var base_some_worried_malaria "Somewhat worried about malaria"
la var base_worried_malaria "Worried about malaria"
la var base_ext_worried_malaria "Extremely worried about malaria"
la var base_not_worried_covid "Not worried about COVID-19"
la var base_some_worried_covid "Somewhat worried about COVID-19"
la var base_worried_covid "Worried about COVID-19"
la var base_ext_worried_covid "Extremely worried about COVID-19"
la var base_malaria_riskperception "Worried/Very worried about malaria"
la var base_covid_riskperception "Worried/Very worried about COVID-19"

replace cluster_population = cluster_population/1000000

gen cluster_cost_per_completion_usd = cluster_cost_per_completion*0.012
gen cluster_cost_per_message_usd = cluster_cost_per_message*0.012
gen cluster_cpm_usd = cluster_cpm*0.012

la var cluster_population "Population (Million)"
la var cluster_pucca "Share living in solid houses"
la var cluster_sleepundernet "Share sleeping under net"
la var cluster_university "Share with university degree"
la var cluster_unemployed "Share unemployed"
la var cluster_malaria2weeks "Share with malaria in last 2 weeks"
la var cluster_malaria5year "Share with malaria in last 5 years"
la var cluster_cost_per_completion_usd "Average ad cost per survey completed (USD)"   
la var cluster_cost_per_message_usd "Average ad cost per survey started (USD)"   
la var cluster_ctr "Average click-through rate (\%)"
la var cluster_cpm_usd "Average cost per mille (USD)"

drop if missing(university) 

save "02_ProcessedData/endline_processed.dta", replace




