* Clean individual-study data
import delim using "01_RAwData/individual_effect.csv", clear

keep if wave!=.
compress

* Outcomes
encode worriedcovid, gen(worried_covid_code)
drop worriedcovid
gen not_worried_covid = (worried_covid_code==3)
gen some_worried_covid = (worried_covid_code==4)
gen worried_covid = (worried_covid_code==5)
gen ext_worried_covid = (worried_covid_code==1)
recode not_worried_covid  some_worried_covid  worried_covid  ext_worried_covid (*=.) if worried_covid_code== 2
drop worried_covid_code

encode worriedmalaria, gen(worried_malaria_code)
drop worriedmalaria
gen not_worried_malaria = (worried_malaria_code==3)
gen some_worried_malaria = (worried_malaria_code==4)
gen worried_malaria = (worried_malaria_code==5)
gen ext_worried_malaria = (worried_malaria_code==1)
recode not_worried_malaria  some_worried_malaria  worried_malaria  ext_worried_malaria (*=.) if worried_malaria_code== 2
drop worried_malaria_code


gen test_malaria = (testmalaria=="Yes")
recode test_malaria (*=.) if testmalaria=="" | testmalaria=="NA"

gen help24_malaria = (timeseekhelpmalaria=="Less than 24 hours")
recode help24_malaria (*=.) if timeseekhelpmalaria =="" | timeseekhelpmalaria =="NA"


gen help_fever = (seekhelpfever=="1")
recode help_fever (*=.) if seekhelpfever=="" | seekhelpfever=="NA"

gen help24_fever = (timeseekhelpfever=="Less than 24 hours")
recode help24_fever (*=.) if timeseekhelpfever =="" | timeseekhelpfever =="NA"



gen would_buy_net = (buymosquitonet=="Yes")
recode would_buy_net (*=.) if buymosquitonet =="NA"

gen know_buy_net = (knowbuymosquitonet=="Yes")
recode know_buy_net (*=.) if knowbuymosquitonet =="NA"

foreach x of varlist malaria4months fever4months numbermalaria4month  hasmosquitonet  {
     destring `x', replace i("NA")
}

replace numbermalaria4months = . if numbermalaria4months>20

destring numbermosquitonet, replace i("NA" ",") force
replace numbermosquitonet = . if numbermosquitonet>20


gen pay_for_net = regexs(2) if regexm(paymosquitonet, "^([^0-9]*)([0-9]+)([^0-9]*)$")
destring pay_for_net, replace
sum pay_for_net,d
replace pay_for_net = . if pay_for_net>r(p99)


gen treat_net = (treatmosquitonet=="Yes")
recode treat_net (*=.) if treatmosquitonet =="NA"


* Time invariant vars
encode distancemedicalcenter, gen(distance)
gen distance_15 = (distance==3)
gen distance_15_30 = (distance==1)
gen distance_30_60 = (distance==2)
recode distance_15 distance_15_30 distance_30_60 (*=.) if distancemedicalcenter == "NA"
drop distancemedicalcenter distance

encode religion, gen(religion_code)
gen hindu = (religion_code==3)
gen muslim = (religion_code==5)
recode hindu muslim (*=.) if religion=="NA"
drop religion*

encode caste, gen(caste_code)

gen caste_general = (caste=="General")
gen caste_obc = (caste=="OBC")
gen caste_other = (caste=="Other")
gen caste_scdalit = (caste=="SC/Dalit")
gen caste_st = (caste=="ST")
recode caste_general caste_obc caste_other caste_scdalit caste_st (*=.) if caste == "NA"

gen secondary = (education =="Secondary")
gen primary = (education =="Primary")
recode caste_general secondary primary (*=.) if education == "NA"

foreach x of varlist age familymembers pucca kutcha semipucca university secondary primary unemployed student female generalcaste forwardcaste backwardcaste hasmosquitonet  hasairconditioning  pregnantwoman  {
     destring `x', replace i("NA")
}


*** INDIVIDUAL LEVEL OUTCOMES

encode treatmentseeking, gen(treatmentseeking_code)
gen seek_treatment_immed = (treatmentseeking_code==3)
recode seek_treatment_immed (*=.) if treatmentseeking=="NA"

gen not_seek_treatment = (treatmentseeking_code==1 | treatmentseeking_code==4)
recode not_seek_treatment (*=.) if treatmentseeking=="NA"

gen seek_treatment = (treatmentseeking_code==2 | treatmentseeking_code==3)
recode seek_treatment (*=.) if treatmentseeking=="NA"

drop treatmentseeking treatmentseeking_code

foreach x of varlist sleepundernet membersbednet membersnotbednet longsleeves sprays  {
     destring `x', replace i("NA")
}

encode talkmalaria, gen(talkmalaria_code)
gen talkmalaria_less1month = (talkmalaria_code==1)
recode talkmalaria_less1month (*=.) if talkmalaria=="NA"


gen talkmalaria_onceaweek = (talkmalaria_code==6|talkmalaria_code==3|talkmalaria_code==2)
recode talkmalaria_onceaweek (*=.) if talkmalaria=="NA"

drop talkmalaria talkmalaria_code



gen knowmnm_yesno = (knowmnm=="Yes")
replace knowmnm_yesno = . if knowmnm=="NA"
drop knowmnm
rename knowmnm_yesno knowmnm

gen admnm_yesno = (admnm=="Yes")
replace admnm_yesno = . if admnm=="NA"
drop admnm
rename admnm_yesno admnm

gen sprays_yesno = (sprays=="Yes")
replace sprays_yesno = . if sprays=="NA"
drop sprays
rename sprays_yesno sprays



destring answer_time_90 answer_time_min, replace i("NA")
keep if answer_time_90/answer_time_min >= 2 & answer_time_90/answer_time_min!=.


keep if age < 90 & age > 17 
keep if familymembers < 30
keep if pucca!=.

gen start = date(survey_start_time, "YMDhms")
format start %td
gen end = date(survey_end_time, "YMDhms")
format end %td

gen treatment_pucca = treatment*pucca 
gen ind_treatment_pucca = ind_treatment*pucca 
gen treatment_ind_treatment = treatment*ind_treatment
gen perc_membersbednet = membersbednet/familymembers


recode perc_membersbednet (*=.)  if perc_membersbednet>1 
keep if sleepundernet!=.  & perc_membersbednet!=.

recode perc_membersbednet sleepundernet (*=.) if hasmosquitonet==0

la var treatment "T"
la var ind_treatment "Individual T"

la var pucca "Solid house"
la var ind_treatment_pucca "Individual T $\times$ Solid house"
la var treatment_pucca "T $\times$ Solid house"




**** BALANCE INDIVIDUAL STUDY

la var pucca "Solid house"
la var treatment "In treatment district"
la var age "Age"
la var semipucca "Semi-solid house"
la var familymembers "N. of HH members"
la var university "Has completed university"
la var secondary "Has completed secondary school"
la var primary "Has completed primary school"
la var unemployed "Unemployed"
la var student "Student"
la var female "Female"
la var caste_general "General caste"
la var caste_obc "OBC caste"
la var caste_scdalit "SC/Dalit caste"
la var caste_st "ST caste"
la var distance_15 "Medical center distance < 15min"
la var distance_15_30 "Medical center distance 15-30min"
la var distance_30_60 "Medical center distance 30-60min"
la var hasairconditioning "Has air conditioning"
la var hasmosquitonet "Has mosquito net"
la var pregnantwoman "Pregnant woman in house"
la var hindu "Hindu"
la var muslim "Muslim"

drop if missing( hasmosquitonet, university, hasairconditioning) 

save "02_ProcessedData/individual_effect_processed.dta", replace






