*** TABLE D1
*** Household-level Predictors of Malaria Incidence
clear all
use "02_ProcessedData/full_panel_processed.dta", replace

gen distance_above_30 = (distance_15==0 & distance_15_30==0)

global ind_predictors pucca familymembers distance_above_30 university backwardcaste hindu 
drop if missing(university) 

keep if post==0
sum  $ind_predictors 

  foreach x of varlist $ind_predictors {
    qui sum `x'
    gen z`x' = (`x'-r(mean))/r(sd)
  } 

la var zpucca "Solid house"
la var zfamilymembers "Household size"
la var zuniversity "Education"
la var zbackwardcaste "Backward caste"
la var zdistance_above_30 "Distance to medical center"
la var zhindu "Hindu"
la var base_malaria5year "Member had malaria last 5 years"
la var base_malaria2week "Member had malaria last 2 weeks"

global zscore_predictors zpucca zfamilymembers zdistance_above_30 zuniversity zbackwardcaste zhindu


*Pairwise correlation
eststo clear
eststo malaria5year: estpost corr base_malaria5year $zscore_predictors
eststo malaria2week: estpost corr base_malaria2weeks $zscore_predictors
esttab malaria5year malaria2week  using "$output/malaria_correlates.tex", replace nogaps not dep  label star(* 0.10 ** 0.05 *** 0.01)  b(3) p(4) compress ///
nonotes note("The table reports the Pearson correlation coefficients between malaria incidence and household characteristics. Baseline responses collected from July 23 to August 19, 2020 are included in the sample. \textit{p-values} in parentheses. \sym{*} \(p<0.1\), \sym{**} \(p<0.05\), \sym{***} \(p<0.01\)")


