*** TABLE D13
** LOGIT MALARIA
clear all
use "02_ProcessedData/full_panel_processed.dta", replace

global ind_controls base_malaria5year familymembers  age university secondary primary unemployed student female caste_obc caste_other caste_scdalit caste_st  distance_15 distance_15_30 distance_30_60 hasairconditioning pregnantwoman hindu muslim hasmosquitonet
sum $ind_controls

global cluster_controls  cluster_lpopulation cluster_university cluster_pucca cluster_unemployed  cluster_malaria2weeks cluster_malaria5year  cluster_cost_per_completion  cluster_ctr cluster_cpm  cluster_cost_per_message
sum $cluster_controls

drop if missing(university)

keep if post == 1 

eststo clear
eststo: logit malaria2weeks treatment i.state_id [pw=waves_answered_malaria2weeks],  cluster(stratumid)
qui margins, dydx(treatment) 
matrix A=  r(b) 
qui sum malaria2weeks  if e(sample) == 1 & treatment == 0  
estadd scalar mean_C =  r(mean) 
estadd scalar sd_C =  r(sd)
estadd scalar mar_eff =  A[1,1]

eststo: logit malaria2weeks treatment pucca i.state_id $ind_controls $cluster_controls [pw=waves_answered_malaria2weeks],  cluster(stratumid)
qui margins, dydx(treatment) 
matrix A=  r(b) 
qui sum malaria2weeks  if e(sample) == 1 & treatment == 0  
estadd scalar mean_C =  r(mean) 
estadd scalar sd_C =  r(sd)
estadd scalar mar_eff =  A[1,1]

eststo: logit malaria2weeks treatment treatment_pucca pucca i.state_id $ind_controls $cluster_controls [pw=waves_answered_malaria2weeks],  cluster(stratumid)
qui margins, dydx(treatment) 
matrix A=  r(b) 
qui sum malaria2weeks  if e(sample) == 1 & treatment == 0  
estadd scalar mean_C =  r(mean) 
estadd scalar sd_C =  r(sd)
estadd scalar mar_eff =  A[1,1]

eststo: logit malaria2weeks treatment i.state_id $ind_controls $cluster_controls if pucca==1 [pw=waves_answered_malaria2weeks],  cluster(stratumid)
qui margins, dydx(treatment) 
matrix A=  r(b) 
qui sum malaria2weeks  if e(sample) == 1 & treatment == 0  
estadd scalar mean_C =  r(mean) 
estadd scalar sd_C =  r(sd)
estadd scalar mar_eff =  A[1,1]


eststo: logit test_malaria treatment i.state_id [pw=waves_answered_test_malaria],  cluster(stratumid)
qui margins, dydx(treatment) 
matrix A=  r(b) 
qui sum test_malaria  if e(sample) == 1 & treatment == 0  
estadd scalar mean_C =  r(mean) 
estadd scalar sd_C =  r(sd)
estadd scalar mar_eff =  A[1,1]

eststo: logit test_malaria treatment pucca i.state_id $ind_controls $cluster_controls [pw=waves_answered_test_malaria],  cluster(stratumid)
qui margins, dydx(treatment) 
matrix A=  r(b) 
qui sum test_malaria if e(sample) == 1 & treatment == 0  
estadd scalar mean_C =  r(mean) 
estadd scalar sd_C =  r(sd)
estadd scalar mar_eff =  A[1,1]

eststo: logit test_malaria treatment treatment_pucca pucca i.state_id $ind_controls $cluster_controls [pw=waves_answered_test_malaria],  cluster(stratumid)
qui margins, dydx(treatment) 
matrix A=  r(b) 
qui sum test_malaria  if e(sample) == 1 & treatment == 0  
estadd scalar mean_C =  r(mean) 
estadd scalar sd_C =  r(sd)
estadd scalar mar_eff =  A[1,1]

eststo: logit test_malaria treatment i.state_id $ind_controls $cluster_controls if pucca==1  [pw=waves_answered_test_malaria],  cluster(stratumid)
qui margins, dydx(treatment) 
matrix A=  r(b) 
qui sum test_malaria  if e(sample) == 1 & treatment == 0  
estadd scalar mean_C =  r(mean) 
estadd scalar sd_C =  r(sd)
estadd scalar mar_eff =  A[1,1]


esttab using "$output/malaria_panel_logit.tex",  star(* 0.1 ** 0.05 *** 0.01) nodep numbers nolegend replace mlabels("Full" "Full" "Full" "Solid house" "Full" "Full" "Full" "Solid house") ///
label se compress nogaps  se(%9.3f) b(%9.3f) stats(N r2_p mar_eff mean_C sd_C, fmt(%9.0f %9.3f %9.3f %9.3f %9.3f) labels("Observations" "Pseudo R-squared" "Marginal effect" "Mean in control" "SD in control" ) layout("\num{@}"))  keep(treatment treatment_pucca pucca) ///
order(treatment treatment_pucca pucca) mgroups("Any member has had malaria" "Any member was tested for malaria", pattern(1 0 0 0 1 0 0 0) prefix(\multicolumn{@span}{c}{) suffix(}) span erepeat(\cmidrule(lr){@span})) ///
indicate("Individual controls=age" "Cluster controls=cluster_university", l($\checkmark$ "")) note("Each observation is an individual. Responses collected from September 1, 2020 to January 31, 2021 are included and are weighted by the number of survey waves the individual completed in this period. In columns (1-4), the dependent variable takes value 1 if the respondent/family member has had malaria at least once in the period under consideration. In columns (5-8), the dependent variable takes value 1 if the respondent/family member with malaria was tested at least once in the period under consideration. T is the treatment indicator. Solid house is a binary indicator equal to 1 if the individual lives in a dwelling made of stones, bricks, cement or concrete. State fixed-effects are included in all specifications. Individual controls include gender, age, household size, caste, education, distance to the medical center, employment and student status, religion, presence of pregnant woman in the house, ownership of bednet and air conditioner, and 5-year malaria incidence at baseline. Cluster controls are reported in Table \ref{tab:districts-balance}. Standard errors are clustered at the district level.  \sym{*} \(p<0.1\), \sym{**} \(p<0.05\), \sym{***} \(p<0.01\)")


