*** TABLE D14
* LOGIT MALARIA
clear all
use "02_ProcessedData/xsection_processed.dta", replace

* controls for individual-level outcome
global ind_controls familymembers hasmosquitonet age university secondary primary unemployed student female caste_obc caste_other caste_scdalit caste_st  distance_15 distance_15_30 distance_30_60 hasairconditioning pregnantwoman hindu muslim 

* cluster-level controls
global cluster_controls cluster_lpopulation cluster_university cluster_pucca cluster_unemployed cluster_malaria2weeks cluster_malaria5year  cluster_cost_per_completion  cluster_ctr cluster_cpm  cluster_cost_per_message 

eststo clear
eststo: logit malaria4months treatment i.state_id,  cluster(stratumid)
qui margins, dydx(treatment) 
matrix A=  r(b) 
qui sum malaria4months if e(sample) == 1 & treatment == 0  
estadd scalar mean_C =  r(mean) 
estadd scalar sd_C =  r(sd)
estadd scalar mar_eff =  A[1,1]

eststo: logit malaria4months treatment pucca $ind_controls $cluster_controls i.state_id,  cluster(stratumid)
qui margins, dydx(treatment) 
matrix A=  r(b) 
qui sum malaria4months  if e(sample) == 1 & treatment == 0  
estadd scalar mean_C =  r(mean) 
estadd scalar sd_C =  r(sd)
estadd scalar mar_eff =  A[1,1]

eststo: logit malaria4months treatment treatment_pucca  pucca $ind_controls $cluster_controls i.state_id,  cluster(stratumid)
qui margins, dydx(treatment) 
matrix A=  r(b) 
qui sum malaria4months  if e(sample) == 1 & treatment == 0  
estadd scalar mean_C =  r(mean) 
estadd scalar sd_C =  r(sd)
estadd scalar mar_eff =  A[1,1]

eststo: logit malaria4months treatment $ind_controls $cluster_controls i.state_id if pucca==1 ,  cluster(stratumid)
qui margins, dydx(treatment) 
matrix A=  r(b) 
qui sum malaria4months  if e(sample) == 1 & treatment == 0  
estadd scalar mean_C =  r(mean) 
estadd scalar sd_C =  r(sd)
estadd scalar mar_eff =  A[1,1]

eststo: logit malaria4months treatment $ind_controls $cluster_controls i.state_id if pucca==0 ,  cluster(stratumid)
qui margins, dydx(treatment) 
matrix A=  r(b) 
qui sum malaria4months  if e(sample) == 1 & treatment == 0  
estadd scalar mean_C =  r(mean) 
estadd scalar sd_C =  r(sd)
estadd scalar mar_eff =  A[1,1]

esttab using "$output/malaria_xsection_logit.tex",  star(* 0.1 ** 0.05 *** 0.01) numbers nolegend replace mlabels("Full" "Full" "Full" "Solid house" "Non-solid house") ///
label se compress nogaps depvars se(%9.3f) b(%9.3f) stats(N r2_p mar_eff mean_C sd_C, fmt(%9.0f %9.3f %9.3f %9.3f %9.3f) labels("Observations" "Pseudo R-squared" "Marginal effect" "Mean in control" "SD in control" ) layout("\num{@}"))  keep(treatment treatment_pucca pucca) ///
order(treatment treatment_pucca pucca) mgroups("1 if any HH member had malaria", pattern(1 0 0 0 0) prefix(\multicolumn{@span}{c}{) suffix(}) span erepeat(\cmidrule(lr){@span})) ///
indicate("Individual controls=age" "Cluster controls=cluster_university", l($\checkmark$ "")) ///
note("Each observation is an individual. Responses collected from January 7, 2021 to March 31, 2021 are included. The dependent variable takes value 1 if the respondent/family member has had malaria since August 2020. T is the treatment indicator. Solid house is a binary indicator equal to 1 if the individual lives in a dwelling made of stones, bricks, cement or concrete. State fixed-effects are included in all specifications. Individual controls include gender, age, household size, caste, education, distance to the medical center, employment and student status, religion, presence of pregnant woman in the house, ownership of bednet and of an air conditioner. Cluster controls are reported in Table \ref{tab:districts-balance}. Standard errors are clustered at the district level. \sym{*} \(p<0.1\), \sym{**} \(p<0.05\), \sym{***} \(p<0.01\)")



