*** TABLE D18
* INCIDENCE (log) COUNT and POISSON
clear all
     use "02_ProcessedData/malaria_subdistrict_processed.dta", replace

     global incid_subdist_controls_post share_urban_post area_post elevation_mean_post elevation_sd_post distance_to_capital_post
     global cluster_controls_post lcluster_population_post cluster_kutcha_post cluster_university_post cluster_malaria5year_post cluster_sleepundernet_post cluster_malaria2weeks_post  

    
     eststo clear
     eststo: reghdfe lmalaria_positive treatment_post con lpopsum_post $incid_subdist_controls_post $cluster_controls_post  state_id#post if share_pop>0.5, absorb(subdistrict month year) cluster(district)
     cap gen sample = e(sample)
     sum malaria_positive if sample == 1 & post == 0 
     estadd scalar mean_pre =  r(mean) 
 
     eststo: reghdfe lmalaria_positive_u treatment_post con  lpopsum_post $incid_subdist_controls_post $cluster_controls_post state_id#post if share_pop>0.5, absorb(subdistrict month year) cluster(district)
     cap gen sample2 = e(sample)
     sum malaria_positive if sample == 1 & post == 0 
     estadd scalar mean_pre =  r(mean)   
    
     eststo: reghdfe lmalaria_positive_r treatment_post con lpopsum_post $incid_subdist_controls_post $cluster_controls_post state_id#post if share_pop>0.5, absorb(subdistrict month year) cluster(district)
     sum malaria_positive if sample == 1 & post == 0 
     estadd scalar mean_pre =  r(mean)   


     *eststo: ppmlhdfe malaria_positive treatment_post con1 lpopsum_post $incid_subdist_controls_post  $cluster_controls_post state_id#post if share_pop>0.5, absorb(district month year) cluster(district) d
     eststo: poisson malaria_positive treatment_post con1 lpopsum_post $incid_subdist_controls_post $cluster_controls_post state_id#post i.district_id i.month i.year if share_pop>0.5, cluster(district)
     estadd scalar pseudoll = e(ll)
     sum malaria_positive if sample == 1 & post == 0 
     estadd scalar mean_pre =  r(mean)   
  
     eststo: poisson malaria_positive_u treatment_post con1 lpopsum_post $incid_subdist_controls_post $cluster_controls_post state_id#post i.district_id i.month i.year if share_pop>0.5 & sample2, cluster(district)
     estadd scalar pseudoll = e(ll)
     *margins, dydx(treatment_post)
     sum malaria_positive if sample == 1 & post == 0 
     estadd scalar mean_pre =  r(mean)   

     eststo: poisson malaria_positive_r treatment_post con1 lpopsum_post $incid_subdist_controls_post $cluster_controls_post state_id#post i.district_id i.month i.year if share_pop>0.5, cluster(district)
     estadd scalar pseudoll = e(ll)
     sum malaria_positive if sample == 1 & post == 0 
     estadd scalar mean_pre =  r(mean)   

     drop sample
     esttab using "$output/incidence_count_dd.tex",  star(* 0.1 ** 0.05 *** 0.01) numbers nomtitles replace mlabels("All" "Urban" "Rural" "All" "Urban" "Rural") ///
     label se compress nogaps depvars se(%9.3f) b(%9.3f) stats(N r2_a pseudoll  mean_pre , fmt(%9.0f  %9.3f %9.1f %9.3f) labels("Observations" "Adj. R-squared" "Log-likelihood" "Mean N. of cases at baseline" ) layout("\num{@}"))  keep(treatment_post ) ///
     order(treatment_post)  mgroups("log(N. of cases)" "Poisson(N. of cases)", pattern(1 0 0 1 0 0) prefix(\multicolumn{@span}{c}{) suffix(}) span erepeat(\cmidrule(lr){@span})) ///
     nolegend indicate("Subdistrict FE=con" "District FE=con1" "Month and year FE=cluster_university_post" "Subdistrict controls=area_post" "District controls=cluster_kutcha_post", l($\checkmark$ "")) note("Notes: Each observation is a subdistrict-month-year. The panel spans April 2020 to May 2021. Post takes value 1 after August 2020, i.e. when the Facebook campaign started. Subdistrict controls include: subdistrict population, area, share of urban population, elevation, terrain ruggedness and distance to the state capital, all interacted with Post. District controls include total population from the Census as well as the shares of baseline survey repondents living in kutcha dwellings, with university degree, sleeping under mosquito nets, and reporting any malaria cases in the last 5 years and 2 weeks, all interacted with Post. All regressions include state indicators interacted with Post. Standard errors are clustered at the district level. Data come from the Indian Health Management Information System. \sym{*} \(p<0.1\), \sym{**} \(p<0.05\), \sym{***} \(p<0.01\)")
    

     *We can interpret the Poisson regression coefficient as follows: for a one unit change in the predictor variable, the difference in the logs of expected counts is expected to change by the respective regression coefficient, given the other predictor variables in the model are held constant.







