*** TABLE D2
*** Household-level Predictors of Malaria Incidence
clear all
use "02_ProcessedData/full_panel_processed.dta", replace

gen distance_above_30 = (distance_15==0 & distance_15_30==0)

global ind_predictors pucca familymembers distance_above_30 university backwardcaste hindu 
drop if missing(university) 

keep if post==0
sum  $ind_predictors 

  foreach x of varlist $ind_predictors {
    qui sum `x'
    gen z`x' = (`x'-r(mean))/r(sd)
  } 

la var zpucca "Solid house"
la var zfamilymembers "Household size"
la var zuniversity "Education"
la var zbackwardcaste "Backward caste"
la var zdistance_above_30 "Distance to medical center"
la var zhindu "Hindu"
la var base_malaria5year "Member had malaria last 5 years"
la var base_malaria2week "Member had malaria last 2 weeks"

global zscore_predictors zpucca zfamilymembers zdistance_above_30 zuniversity zbackwardcaste zhindu


* Predictors
eststo clear
eststo: reghdfe base_malaria5year $zscore_predictors, noabsorb cluster(stratumid)
qui sum base_malaria5year if e(sample) == 1 
estadd scalar mean_all =  r(mean) 
sum base_malaria5year if e(sample) == 1 & pucca==1
estadd scalar mean_pucca =  r(mean) 
sum base_malaria5year if e(sample) == 1 & pucca==0
estadd scalar mean_nopucca =  r(mean) 

eststo:  reghdfe base_malaria2weeks $zscore_predictors, noabsorb cluster(stratumid)
qui sum base_malaria2weeks if e(sample) == 1 
estadd scalar mean_all =  r(mean) 
sum base_malaria2weeks if e(sample) == 1 & pucca==1
estadd scalar mean_pucca =  r(mean) 
sum base_malaria2weeks if e(sample) == 1 & pucca==0
estadd scalar mean_nopucca =  r(mean) 

eststo:  reghdfe zpucca zfamilymembers zdistance_above_30 zuniversity zbackwardcaste zhindu, noabsorb cluster(stratumid)

esttab using "$output/malaria_predictors.tex",  star(* 0.1 ** 0.05 *** 0.01) nomti numbers nolegend replace  mgroups("\begin{tabular}{@{}c@{}}Member had malaria \\ last 5 years\end{tabular}" "\begin{tabular}{@{}c@{}}Member had malaria \\ last 2 weeks\end{tabular}" "Solid House", pattern(1 1 1) prefix(\multicolumn{@span}{c}{) suffix(}) span erepeat(\cmidrule(lr){@span})) ///
label se compress nogaps depvars se(%9.3f) b(%9.3f) stats(N r2_a mean_all mean_pucca mean_nopucca, fmt(%9.0f  %9.3f %9.3f %9.3f %9.3f) labels("Observations" "Adj. R-squared" "Mean of Y" "Mean of Y: Solid house" "Mean of Y: Non/Semi-solid house" ) layout("\num{@}")) ///
note("The table reports the OLS coefficients from a multivariate regression of malaria incidence on standardized household characteristics (columns 1-2) and of dwelling type on the remaining household characteristics (column 3). Baseline responses collected from July 23 to August 19, 2020 are included in the sample. Standard errors are clustered at the district level. \sym{*} \(p<0.1\), \sym{**} \(p<0.05\), \sym{***} \(p<0.01\)")





