*** TABLES D5
* SUMMARY STATS BY DWELLING TYPE
clear all
use "02_ProcessedData/full_panel_processed.dta", replace

global ind_controls treatment tot_waves_answered female familymembers age university secondary primary unemployed student caste_obc caste_scdalit caste_st caste_general  distance_15 distance_15_30 distance_30_60 hasmosquitonet hasairconditioning pregnantwoman hindu muslim base_malaria5year base_malaria_riskperception base_covid_riskperception 
drop if missing(university) 
sum $ind_controls if post 

global ind_outcomes base_sleepundernet base_perc_membersbednet base_longsleeves base_used_spray base_malaria2weeks base_fever2weeks base_test_malaria base_help_fever

la var base_perc_membersbednet "Share HH members using bednet previous night"

keep if post == 1

* Individual level vars
* First test of differences
         local i = 1
          foreach var in $ind_controls {
              qui reg `var' pucca, cluster(stratumid)
              outreg, keep(pucca)  rtitle("`: var label `var''") stats(p) sdec(3) ///
                  noautosumm store(row`i')  
              outreg, replay(diff_ind) append(row`i') ctitles("",Difference ) ///
                  store(diff_ind) note("")
              local ++i
          }
          outreg, replay(diff_ind) store(diff)

          outreg, replay(new) addrows("\hline \textbf{Panel A: Covariates}" "" "") addrtc(1) store(title)
          outreg, replay(title) addrows("" "" "\\[-5ex]") store(title)
          outreg, replay(title) addrows("N. of respondents" "" "") store(title)


          outreg, replay(title) append(diff) store(diff)


          * Then Summary statistics
          local count: word count $ind_controls
          mat sumstat = J(`count',3,.)

          local i = 1
          foreach var in $ind_controls {
              qui: summarize `var' if pucca==1
              mat sumstat[`i',1] = r(mean)
              local vart = r(Var)
              qui: summarize `var' if pucca==0 
              mat sumstat[`i',2] = r(mean)
              local varcon = r(Var)
              mat sumstat[`i',3] = (sumstat[`i',1]-sumstat[`i',2])/(sqrt(`vart'+`varcon'))
              local i = `i' + 1
          }
          frmttable, statmat(sumstat) store(sumstat) sfmt(f,f,f) sdec(3)
          
          sum female if pucca==1
              local nt = r(N) 
          sum female if pucca==0
              local ncon = r(N)


          outreg, replay(new) addrows(""\"")  store(title)
          outreg, replay(title) addrows( "`nt'" "`ncon'" ) addrtc(0) store(title)

          outreg, replay(title) append(sumstat) store(sumstat)

          outreg, replay(sumstat) merge(diff) store(diff_ind) addrows("\hline \hline \\[-5.5ex]")



* Outcomes 
* First test of differences

         local i = 1
          foreach var in $ind_outcomes {
              qui reg `var' pucca, cluster(stratumid)
              outreg, keep(pucca)  rtitle("`: var label `var''") stats(p) sdec(3) ///
                  noautosumm store(row`i')  
              outreg, replay(diff_clu) append(row`i') ctitles("",Difference ) ///
                  store(diff_clu) note("")
              local ++i
          }
          outreg, replay(diff_clu) store(diff)

          outreg, replay(new) addrows("\textbf{Panel B: Outcomes}" "" "") addrtc(1) store(title)
          outreg, replay(title) addrows("" "" "\\[-5ex]") store(title)
          outreg, replay(title) append(diff) store(diff)


     * Then Summary statistics
          local count: word count $ind_outcomes
          mat sumstat = J(`count',3,.)

          local i = 1
          foreach var in $ind_outcomes {
              qui: summarize `var' if pucca==1 
              mat sumstat[`i',1] = r(mean)
              local vart = r(Var)
              qui: summarize `var' if pucca==0 
              mat sumstat[`i',2] = r(mean)
              local varcon = r(Var)
              mat sumstat[`i',3] = (sumstat[`i',1]-sumstat[`i',2])/(sqrt(`vart'+`varcon'))
              local i = `i' + 1
          }
          frmttable, statmat(sumstat) store(sumstat) sfmt(f,f,f) sdec(3)
     

          sum cluster_university if pucca==1 
              local nt = r(N) 
          sum cluster_university if pucca==0 
              local ncon = r(N)

          outreg, replay(new) addrows(""\"")  store(title)

          outreg, replay(title) append(sumstat) store(sumstat)
          outreg, replay(sumstat) merge(diff) store(diff_clu) 
     
          outreg, replay(diff_ind) append(diff_clu) store(tab) addrows("\hline \\[-5.1ex]")


         outreg using "$output/panel_sample_bydwelling.tex", ///
         replay(tab) tex nocenter fragment plain replace ///
         ctitles("","Solid House","Non/Semi-solid House","Normalized", "\it T-test" \"","\it Mean","\it Mean", Difference, "(\it p-value)" \"", "(1)","(2)","(3)","(4)") ///
         note("The table reports covariates and outcomes measured at baseline for individuals' and households' in the experience sample, by dwelling type. The normalized difference is the difference between the two means divided by the square root of the sum of the variances of the two variables.")
     






