*** TABLE D7
**** DESCRIPTIVE STATS FOR SUBDISTRICT-LEVEL DATA
clear all
     use "02_ProcessedData/malaria_subdistrict_processed.dta", replace

     **** SUMMARY STATS
     eststo clear
     estpost summarize  malaria_positive incidence_rate tests_percapita malaria_rate_pc pop_k if post == 0 , listwise
     eststo all_sample
     gdistinct subdistrict_id if  tests_percapita!=.
     estadd scalar obs =  r(ndistinct)   

     preserve 
     la var malaria_positive_u "N. of cases"
     la var uincidence_rate "N. of cases per M people"
     la var utests_percapita "N. of tests per M people"
     la var malaria_rate_u_pc "Test positivity rate (\%)"
     la var pop_k "Population (Thousands)"

     keep uincidence_rate malaria_positive_u utests_percapita malaria_rate_u_pc post share_pop pop_k date subdistrict_id
     rename  uincidence_rate incidence_rate
     rename malaria_positive_u malaria_positive
     rename utests_percapita tests_percapita
     rename malaria_rate_u_pc malaria_rate_pc
     estpost summarize  malaria_positive incidence_rate tests_percapita malaria_rate pop_k if post == 0 , listwise 
     eststo urban_sample
    gdistinct subdistrict_id if tests_percapita!=.
     estadd scalar obs =  r(ndistinct) 
     restore

     preserve 
     la var malaria_positive_r "N. of cases"
     la var rincidence_rate "N. of cases per M people"
     la var rtests_percapita "N. of tests per M people"
     la var malaria_rate_r_pc "Test positivity rate (\%)"
     la var pop_k "Population (Thousands)"

     keep rincidence_rate malaria_positive_r rtests_percapita malaria_rate_r_pc post share_pop pop_k date subdistrict_id
     rename  rincidence_rate incidence_rate
     rename malaria_positive_r malaria_positive
     rename rtests_percapita tests_percapita
     rename malaria_rate_r_pc malaria_rate_pc
     estpost summarize  malaria_positive incidence_rate tests_percapita malaria_rate pop_k if post == 0 , listwise 
     eststo rural_sample
    gdistinct subdistrict_id if tests_percapita!=.
     estadd scalar obs =  r(ndistinct)   
     restore


     estpost summarize  malaria_positive incidence_rate tests_percapita malaria_rate_pc pop_k if post == 0 & share_pop>0.5, listwise
     eststo all_subsample
     gdistinct subdistrict_id if  tests_percapita!=. & share_pop>0.5
     estadd scalar obs =  r(ndistinct) 
     
     preserve 
     la var malaria_positive_u "N. of cases"
     la var uincidence_rate "N. of cases per M people"
     la var utests_percapita "N. of tests per M people"
     la var malaria_rate_u_pc "Test positivity rate (\%)"
     la var pop_k "Population (Thousands)"
     keep uincidence_rate malaria_positive_u utests_percapita malaria_rate_u_pc post share_pop pop_k date subdistrict_id
     rename  uincidence_rate incidence_rate
     rename malaria_positive_u malaria_positive
     rename utests_percapita tests_percapita
     rename malaria_rate_u_pc malaria_rate_pc
     estpost summarize  malaria_positive incidence_rate tests_percapita malaria_rate pop_k if post == 0 & share_pop>0.5, listwise 
     eststo urban_subsample
     gdistinct subdistrict_id if  tests_percapita!=. & share_pop>0.5
     estadd scalar obs =  r(ndistinct)
     restore

     preserve 
     la var malaria_positive_r "N. of cases"
     la var rincidence_rate "N. of cases per M people"
     la var rtests_percapita "N. of tests per M people"
     la var malaria_rate_r_pc "Test positivity rate (\%)"
     la var pop_k "Population (Thousands)" 
     keep rincidence_rate malaria_positive_r rtests_percapita malaria_rate_r_pc post share_pop pop_k date subdistrict_id
     rename  rincidence_rate incidence_rate
     rename malaria_positive_r malaria_positive
     rename rtests_percapita tests_percapita
     rename malaria_rate_r_pc malaria_rate_pc
     estpost summarize  malaria_positive incidence_rate tests_percapita malaria_rate pop_k if post == 0 & share_pop>0.5, listwise 
     eststo rural_subsample
     gdistinct subdistrict_id if  tests_percapita!=. & share_pop>0.5
     estadd scalar obs =  r(ndistinct)  
     restore
     

     esttab all_sample urban_sample rural_sample all_subsample urban_subsample rural_subsample using "$output/dd_sample.tex", ///
     cells(mean(fmt(2))) label collabels(,none) nonumber nomtitles nodep mlabels("All health facilities" "Urban" "Rural" "All health facilities" "Urban" "Rural") replace ///
     mgroups("All subdistricts" "Subdistricts with >50\% population within circles", pattern(1 0 0 1 0 0) prefix(\multicolumn{@span}{c}{) suffix(}) span erepeat(\cmidrule(lr){@span})) ///
     stats(obs, fmt(%9.0f) labels("N. of subdistricts") layout("\num{@}")) note("Notes: The table reports the means of the variables across the different subsamples. Only subdistricts with non-missing entries are included. Subdistricts where the number of positive malaria cases was larger than the number of malaria tests conducted are dropped. Not all the subdistricts contain both urban and rural health facilities. Urban health facilities are present only in a subset of subdistricts.")

     
